/**
 * Titanium Windows Native Module - module.test3
 *
 * Copyright (c) 2015 by Appcelerator, Inc. All Rights Reserved.
 * Licensed under the terms of the Apache Public License.
 * Please see the LICENSE included with this distribution for details.
 */
#include "ComAppcModuleTest3.hpp"
#include "Titanium/detail/TiImpl.hpp"
#include "TitaniumWindows/LogForwarder.hpp"
#include "TitaniumWindows/WindowsMacros.hpp"
#include "TitaniumWindows/Utility.hpp"

using namespace Windows::ApplicationModel::Background;

namespace ComAppcModuleTest3
{
	/*
	 * @class
	 * @discussion This is background task for BackgroundService.
	 *             EntryPoint: ComAppcModuleTest3.BackgroundServiceTask
	 *
	 *   Don't remove this class because Windows Store submission process requires
	 *   at least one C++/CX class in winmd. (TIMOB-20192)
	 * 
	 *   ## Usage: (JavaScript) ##
	 *     var task = Ti.App.Windows.BackgroundService.registerTimerTask(
	 *                'ComAppcModuleTest3.BackgroundServiceTask', 15, false);
	 *
	 *   ## tiapp.xml ##
	 *   <windows>
	 *     <manifest>
	 *       <Extensions>
	 *         <Extension Category="windows.backgroundTasks" EntryPoint="ComAppcModuleTest3.BackgroundServiceTask">
	 *           <BackgroundTasks>
	 *             <Task Type="timer" />
	 *           </BackgroundTasks>
	 *         </Extension>
	 *       </Extensions>
	 *     </manifest>
	 *   </windows>
	 *
	 */
	[Windows::Foundation::Metadata::WebHostHidden]
	public ref class BackgroundServiceTask sealed : public Windows::ApplicationModel::Background::IBackgroundTask
	{
	public:
		virtual void Run(Windows::ApplicationModel::Background::IBackgroundTaskInstance^ taskInstance);
	};

	/*
	 * BackgroundServiceTask::Run implementation
	 */
	void BackgroundServiceTask::Run(IBackgroundTaskInstance^ taskInstance)
	{
		const auto deferral = taskInstance->GetDeferral();

		//
		// BACKGROUND TASK: IMPLEMENT SOMETHING USEFUL HERE :)
		//

		// Inform this task has finished
		deferral->Complete();
	}
}

namespace Com
{
	namespace Appc
	{
		namespace Module
		{
			Test3::Test3(const JSContext& js_context) TITANIUM_NOEXCEPT
				: Titanium::Module(js_context, "com.appc.module.test3")
			{
				TITANIUM_LOG_DEBUG("Test3::ctor Initialize");
			}

			void Test3::postInitialize(JSObject& js_object)
			{
			}

			void Test3::postCallAsConstructor(const JSContext& js_context, const std::vector<JSValue>& arguments)
			{
			}

			void Test3::JSExportInitialize()
			{
				JSExport<Test3>::SetClassVersion(1);
				JSExport<Test3>::SetParent(JSExport<Titanium::Module>::Class());
				TITANIUM_ADD_FUNCTION(Test3, testUsageInfo);
				TITANIUM_ADD_FUNCTION(Test3, testLogonCore);
			}

			TITANIUM_FUNCTION(Test3, testUsageInfo)
			{
#if defined(IS_WINDOWS_10)
				const auto usage = ref new SAP::Usage::UsageInfo();
				return get_context().CreateString(TitaniumWindows::Utility::ConvertString(usage->ToString()));
#else
				return get_context().CreateString("hello world");
#endif
			}

			TITANIUM_FUNCTION(Test3, testLogonCore)
			{
#if defined(IS_WINDOWS_10)
				return get_context().CreateString(TitaniumWindows::Utility::ConvertString(SAP::Logon::Core::RegistrationServiceVersion::V4.ToString()));
#else
				return get_context().CreateString("hello world");
#endif
			}

		}
	}
}
